# Copyright (c) 2016-2018 Braintech Sp. z o.o. [Ltd.] <http://www.braintech.pl>
# All rights reserved
import os
import sys

from devops import devpi
from devops.devpi import _get_package_version, _index_arg
from fabric import context_managers, api

from setup import INSTALL_REQUIREMENTS

try:
    import devops  # noqa
except ImportError:
    print("devops module not found! Not all commands are available!\n"
          "Install devops module using https://pip.braintech.pl/obci/stable", file=sys.stderr)
else:
    from devops.ci import install
    from devops.ci import output

    sys.stdout = output.unbuffered_stdout

APT_REQUIREMENTS_PACKAGES = {
    'common': ['git', 'python3-pip'],
    'build': [],
    'codestyle': ([]),
    'deploy': ['uuid-runtime'],
    'test': ['xfwm4 xvfb xorg-dev libglu1-mesa libgl1-mesa-dev libxinerama1 libxcursor1']

}


def setup_stage(name):
    apt_requirements = APT_REQUIREMENTS_PACKAGES['common'] + APT_REQUIREMENTS_PACKAGES[name]
    requirements = [(install.APT, r) for r in apt_requirements]
    requirements += [(install.PIP3, r) for r in INSTALL_REQUIREMENTS.get(name, [])]
    install.install_aggregated(requirements)


def generate_docker_file(path):
    docker_file = """
FROM devops:1804
RUN pip3 install --index {DEVPI_URL}/obci/dev devops~={DEVOPS_VERSION} setuptools
RUN devpi_use {DEVPI_URL}/obci/dev
COPY setup.py ./setup.py
COPY fabfile.py ./fabfile.py
RUN fab setup_stage:test -f ./fabfile.py
RUN pip3 uninstall -y devops""".format(**os.environ)
    open(path, "w").write(docker_file)


def _push_no_delete(path, index, from_index=None):
    package, version = _get_package_version(path)
    with context_managers.lcd(path):
        api.local("devpi push {package}=={version} {index} {from_index}".format(
            package=package, index=index, version=version, from_index=_index_arg(from_index)
        ))


def deploy_packages_to_installer():
        devpi_index = devpi.current_index()['name']
        from_index = devpi.temp_name(devpi_index)
        _push_no_delete('.', 'obci/installers', from_index)
