# Copyright (c) 2016-2018 Braintech Sp. z o.o. [Ltd.] <http://www.braintech.pl>
# All rights reserved.

import os.path
import pytest

import braintech.drivers.blinker.blinker_preview as blinker_preview
from braintech.drivers.blinker.blinker import run_demo
from braintech.drivers.blinker.fat_blinker import FatBlinker, Fat16Blinker
from braintech.drivers.blinker.fuw_blinker import FUWBlinker
from braintech.drivers.blinker.slim_blinker import SlimBlinker, Slim16Blinker

blinker_preview_path = os.path.realpath(blinker_preview.__file__)

NEW_BLINKERS = [FatBlinker, SlimBlinker]


@pytest.fixture(params=NEW_BLINKERS + [Fat16Blinker, Slim16Blinker, FUWBlinker],
                ids=lambda c: c.__name__)
def BlinkerClass(request):
    return request.param


@pytest.fixture(params=NEW_BLINKERS, ids=lambda c: c.__name__)
def NewBlinker(request):
    return request.param


def test_all_blinkers(BlinkerClass):
    blinker = BlinkerClass(port=None, preview=True)
    run_demo(blinker, 0.5)
    del blinker


def test_field_defines(NewBlinker):
    blinker = NewBlinker(port=None, preview=True)
    blinker.set_number_of_fields(blinker.CHANNELS_NUMBER)
    for i in range(blinker.CHANNELS_NUMBER):
        blinker.define_field(i, [i])
        blinker.set_frequency(i, 0.1, i * blinker.MAX_PHASE / blinker.CHANNELS_NUMBER)
        blinker.enable(i)
    blinker.start()
    blinker.flush()
    del blinker
