# import amp driver
from braintech.drivers.perun32.amplifier import Perun32Amplifier
import numpy as np

# import blinker driver
from braintech.drivers.blinker.blinker import get_blinker
import time

# Find amps
amps = Perun32Amplifier.get_available_amplifiers()
if len(amps) < 1:
    raise Exception("Amplifier not connected")
amp = Perun32Amplifier(amps[0])

amp.sampling_rate = 500



def samples_to_microvolts(samples):  # amplifier streams integers straight from ADC, we need to change them to micro volts
    return samples * gains + offsets
    

# find blinker
blinker = get_blinker(None, preview=True)  # will fail if no blinker is connected or driver not properly installed
print(blinker)
# lets setup 3 logical fields manually, using 2 top-left blocks, and last segmment (verti

blinker.set_number_of_fields(3)
blinker.define_field(0, [0, 1, 2, 3, 4, 5, 6])
blinker.define_field(1, [8, 9, 10, 11, 12, 13, 14])
blinker.define_field(2, [319,])

# alternatively setting up 3 logical fields using automatic physical field detection - it tries capture all physical fields beneath the rectangular logical one
# field 1 positioned 5 percent of sreen size from top left corner, sized 10% of screen size
segments1 = blinker.get_segments_for_rect(x=0.05, y=0.05, width=0.10, height=0.10, in_pixels=False)

# another field to the right
segments2 = blinker.get_segments_for_rect(x=0.25, y=0.05, width=0.10, height=0.10, in_pixels=False)
segments3 = blinker.get_segments_for_rect(x=0.40, y=0.05, width=0.10, height=0.10, in_pixels=False)

blinker.define_field(0, segments1)
blinker.define_field(1, segments2)
blinker.define_field(2, segments3)

# enable the fields for blinking
blinker.enable(2)
blinker.enable(1)
blinker.enable(0)

blinker.set_intensity(1.0)
# program field 1 to blink with rectangular function (fill 50/50) at frequency 2 Hz
blinker.set_rectangular_function(0, 1.0, 0.5, luminosity=1)
# program field 2 to blink with sinefunction (fill 50/50) at frequency 3 Hz
blinker.set_sine_function(1, 1.0, luminosity=1)

# you can also have it phase delayed
blinker.set_sine_function(2, 1.0, 180, luminosity=1)

start_timestamp = blinker.start()
print("BLinking using blinker")

time.sleep(10)

blinker.stop()
# you can set their brightness and stop blinking

blinker.set_still(0, 1)
blinker.set_still(1, 0.1)
blinker.set_still(2, 0.0)

# you can receive timestamps of the configured sequence start. Which is aligned with signal timestamp clock
still_timestamp = blinker.start()

amp.start_sampling()
gains = np.array(amp.current_description.channel_gains)
offsets = np.array(amp.current_description.channel_offsets)
while True:
    # getting 16 samples in a package
    # I encourage you to use IPython to experiment with the class
    packet = amp.get_samples(16)
    print(samples_to_microvolts(packet.samples))
    print(packet.ts[0])
    print(packet.samples.shape, amp.current_description.channel_names)
