# Copyright (c) 2016-2018 Braintech Sp. z o.o. [Ltd.] <http://www.braintech.pl>
# All rights reserved.

"""Package providing small saw signal veryfier."""
from braintech.obci.core.broker import messages
from braintech.obci.core.broker.peer import Peer
from braintech.obci.core.utils.signal_generators import SawVerifier


__all__ = ('DummySignalVerifierPeer',)


class DummySignalVerifierPeer(Peer):
    """Receives signal generated by `DummySignalReceiverPeer` and verifies saw signal."""

    def __init__(self, *args, receiver_peer_name='Receiver', **kwargs):
        """
        Class which verifies saw.

        :param args: args for Peer class
        :param receiver_peer_name: To whom listen for signal
        :param kwargs: kwargs for Peer class
        """
        super().__init__(*args, **kwargs)
        self._saw_verifier = SawVerifier()
        self.signal_ok = None
        self._receiver_peer_name = receiver_peer_name

    async def _connections_established(self):
        await super()._connections_established()
        self.subscribe_for_specific_msg_subtype(messages.SignalMessage,
                                                self._receiver_peer_name, self.handle_signal_message)

    async def handle_signal_message(self, msg) -> None:
        """
        Coroutine accepting signal message.

        Coroutine which works forever, receives signal and
        raises exception when there something wrong with it.

        :param msg: Message with signal
        :return: None
        :raises: Exception, when signal is distorted
        """
        for data in msg.data.samples:
            try:
                self._saw_verifier.verify_next(data[2])
                if self.signal_ok is None:
                    self.signal_ok = True
            except Exception:
                self.signal_ok = False
