# Copyright (c) 2016-2018 Braintech Sp. z o.o. [Ltd.] <http://www.braintech.pl>
# All rights reserved.

"""Module defines single :method 'to_string' converting p_object to string."""
import warnings


def to_string(p_object):
    """Convert p_object to string. Convert strings to strings, floats to string with accurate precision etc."""
    if isinstance(p_object, float):
        return repr(p_object)  # preserve precision
    else:
        return str(p_object)


class property_no_longer_public(property):
    def __init__(self, *args, public_name, private_name, **kwargs):
        super().__init__(*args, **kwargs)
        self._private_name = private_name
        self._public_name = public_name

    def __get__(self, instance, owner):
        warnings.warn(
            'Attribute "{}" will be removed in the future'
                .format(self._public_name),
            DeprecationWarning,
            stacklevel=2,
        )
        return getattr(instance, self._private_name)

    def __set__(self, instance, value):
        warnings.warn(
            'Attribute "{}" will be removed in the future'
                .format(self._public_name),
            DeprecationWarning,
            stacklevel=2,
        )
        setattr(instance, self._private_name, value)
