# Copyright (c) 2016-2018 Braintech Sp. z o.o. [Ltd.] <http://www.braintech.pl>
# All rights reserved.

import cv2

from . import settings


class NoCameraAvailable(Exception):
    pass


def images_generator():
    camera = _pick_last_camera()
    camera.set(cv2.CAP_PROP_FRAME_WIDTH, settings.RESOLUTION[0])
    camera.set(cv2.CAP_PROP_FRAME_HEIGHT, settings.RESOLUTION[1])
    while True:
        success, image = camera.read()
        assert success, "There was a problem getting image from the camera."
        yield cv2.cvtColor(src=image, code=cv2.COLOR_BGR2GRAY)


def _pick_last_camera():
    for i in [2, 1, 0]:
        camera = cv2.VideoCapture(i)
        success, _ = camera.read()
        if success:
            return camera
    else:
        raise NoCameraAvailable
