# Copyright (c) 2016-2018 Braintech Sp. z o.o. [Ltd.] <http://www.braintech.pl>
# All rights reserved.

import math

LEFT_EYE_POINTS_INDEXES = range(36, 41 + 1)
RIGHT_EYE_POINTS_INDEXES = range(42, 47 + 1)


def are_open(face_shape, eyes_open_borderline_ratio):
    left_ratio = _calculate_eye_ratio(face_shape, LEFT_EYE_POINTS_INDEXES)
    right_ratio = _calculate_eye_ratio(face_shape, RIGHT_EYE_POINTS_INDEXES)
    return min([left_ratio, right_ratio]) >= eyes_open_borderline_ratio


def _calculate_eye_ratio(face_shape, eye_indexes):
    left_eye_points = [face_shape.part(i) for i in eye_indexes]
    grouped_left_points = _group_eye_points_for_size_calculation(left_eye_points)
    left_eye_shape = _calculate_eye_shape(grouped_left_points)
    return left_eye_shape['height'] / left_eye_shape['width']


def _group_eye_points_for_size_calculation(ordered_points):
    return {
        'for_width': [ordered_points[0], ordered_points[3]],
        'for_height': [
            [ordered_points[1], ordered_points[5]],
            [ordered_points[2], ordered_points[4]],
        ],
    }


def _calculate_eye_shape(grouped_points):
    width = _calculate_2d_distance(*grouped_points['for_width'])
    height = max(_calculate_2d_distance(*pair)
                 for pair in grouped_points['for_height'])
    return {'width': width, 'height': height}


def _calculate_2d_distance(point1, point2):
    distance_x = abs(point1.x - point2.x)
    distance_y = abs(point1.y - point2.y)
    return math.sqrt(distance_x ** 2 + distance_y ** 2)
