# Copyright (c) 2016-2018 Braintech Sp. z o.o. [Ltd.] <http://www.braintech.pl>
# All rights reserved
import pathlib

from setuptools import find_packages, setup
from setuptools.command.develop import develop
from setuptools.command.install import install
import versioneer

INSTALL_REQUIREMENTS = {
    'install': [
        'braintech-obci-core',
        'obci-p300',
        'dlib~=19.10.0',
        'opencv-python~=3.4.0.12',
        'numpy',
        'scipy'
    ],
    'setup': [

    ]
}


def _add_post_run(cmd, post_run_command):
    class PostRunCommand(cmd):
        def run(self):
            super().run()
            post_run_command()

    return PostRunCommand


def _download_shape_predictor():
    import bci_artifacts.install
    bci_artifacts.install.download_face_predictor()


cmd_classes = {}
cmd_classes['develop'] = _add_post_run(cmd_classes.get('develop', develop), _download_shape_predictor)
cmd_classes['install'] = _add_post_run(cmd_classes.get('install', install), _download_shape_predictor)

if __name__ == '__main__':
    setup(
        name='bci-artifacts',
        version=versioneer.get_version(),
        cmdclass=cmd_classes,
        zip_safe=False,
        author='BrainTech',
        author_email='admin@braintech.pl',
        description='BCI Panels.',
        packages=find_packages(
            exclude=[
                'test',
                'scripts',
            ]
        ),
        include_package_data=True,
        install_requires=INSTALL_REQUIREMENTS['install'],
        setup_requires=INSTALL_REQUIREMENTS['setup'],
        extras_require=INSTALL_REQUIREMENTS,
        entry_points={
            'console_scripts': [
                'install_bci_artifacts = bci_artifacts.install:run',
            ]
        }
    )
