import tempfile

from bci_ssvep.ssvep_random_amplifier_peer import BaseSSVEPRandomAmplifierPeer
from obci_demo import amplifiers
from obci_demo.amplifiers import register_demo_amplifier
from obci_demo.config_generation import validate_amplifier_id, InvalidSamplingRateSelected, \
    get_amplifier_config, get_wisdom_path


class InvalidFieldFrequenciesSelected(Exception):
    pass


calibration_files_dir = tempfile.TemporaryDirectory(prefix='bci_demo-')


@register_demo_amplifier('RandomAmplifierWithSinePeer')
class SSVEPRandomAmplifierPeer(amplifiers.PreconfiguredBaseAmplifierPeer, BaseSSVEPRandomAmplifierPeer):
    pass


def generate(amplifier_id, analysis_channels, montage_channels,
             exclude_from_montage, sampling_rate, filter_frequency,
             field_freqs_min, field_freqs_max, wisdom_path=None):
    amplifier = validate_amplifier_id(amplifier_id)
    if sampling_rate is None:
        sampling_rates = amplifier['sampling_rates']
        for s in sampling_rates:
            if s > field_freqs_max * 4:
                sampling_rate = s
                break
        else:
            raise InvalidSamplingRateSelected("No sampling rate available for these stimulation frequencies")
    amplifier_config = get_amplifier_config(amplifier_id, analysis_channels, montage_channels, sampling_rate)
    validate_field_freqs_boundaries(field_freqs_min, field_freqs_max)
    return {
        'amplifier': amplifier_config,
        'ssvep': {
            'channels_for_montage': montage_channels,
            'channels_for_classification': analysis_channels,
            'wisdom_path': get_wisdom_path(wisdom_path, 'bci_demo_ssvep'),
            'field_freqs_search': [field_freqs_min, field_freqs_max],
            'field_statuses': ['YES', 'NO'],
            'exclude_from_montage': exclude_from_montage,
            'power_line_freq': filter_frequency,
            'channel_gains': amplifier_config['gains'],
            'channel_offsets': amplifier_config['offsets'],
        },
        'controller': {
            'title': "BCI SSVEP"
        }
    }


def validate_field_freqs_boundaries(field_freqs_min, field_freqs_max):
    if not (field_freqs_min + 2 <= field_freqs_max):
        raise InvalidFieldFrequenciesSelected('Minimal frequency must be smaller '
                                              'than the max by at least 2.')
