import argparse
import logging

from bci_demo_ssvep import config_generation
from bci_demo_ssvep import demo_ssvep_app
from obci_demo import entrypoint

logger = logging.getLogger(__name__)


class AnotherInstanceIsRunning(Exception):
    pass


def run():
    entrypoint.run(demo_ssvep_app.run, get_argument_parser())


def get_argument_parser():
    parser = entrypoint.get_argument_parser()
    parser.add_argument("--field-freqs-min", default=15, type=int, choices=range(1, 48),
                        help="Minimal SSVEP blinking frequency.")
    parser.add_argument("--field-freqs-max", default=45, type=int, choices=range(3, 50),
                        help="Maximal SSVEP blinking frequency.")
    parser.generate_config = generate_config
    return parser


def generate_config(args: argparse.Namespace) -> dict:
    config = config_generation.generate(
        amplifier_id=args.amplifier_id,
        analysis_channels=args.analysis_channels,
        montage_channels=args.montage_channels,
        exclude_from_montage=args.exclude_from_montage,
        sampling_rate=args.sampling_rate,
        filter_frequency=args.filter,
        field_freqs_min=args.field_freqs_min,
        field_freqs_max=args.field_freqs_max,
        wisdom_path=args.wisdom_path,
    )
    if args.start:
        config['start'] = True
    return config


if __name__ == '__main__':
    run()
