# Copyright (c) 2016-2018 Braintech Sp. z o.o. [Ltd.] <http://www.braintech.pl>
# All rights reserved
from logging import getLogger
from queue import Empty

from bci_ssvep.logic.ssvep_analysis import SSVEPAnalysis
from obci_p300.utils import ArrayCascadeFilter

LOGGER = getLogger(__name__)


class SimpleSSVEPAnalysis(SSVEPAnalysis):

    def __init__(self, stimulation_frequencies,
                 sample_rate,
                 channels_for_classification,
                 **kwargs
                 ):
        super().__init__(stimulation_frequencies=stimulation_frequencies,
                         sample_rate=sample_rate,
                         channels_for_classification=channels_for_classification,
                         channels_for_montage=[],
                         exclude_from_montage=[],
                         channel_names=channels_for_classification,
                         signal_goodness_reporters=None,
                         **kwargs)

        # filtering is already done somewher else
        self.baseline_filter = ArrayCascadeFilter()

    def ssvep_montage(self, raw_data):
        # montage is already done somewhere else
        return raw_data

    def start_calibration(self, calibration_type):
        if calibration_type == -1:
            calibration_freq = 0
        else:
            calibration_freq = self.stimulation_freqs[calibration_type]
        LOGGER.debug("Start calibration for %s Hz", calibration_freq)
        self._handle_calibration_start(str(calibration_freq))

    def end_calibration(self):
        LOGGER.info('Sample counter {}'
                    .format([(i[0], len(i[1]))
                             for i in self.calibration_buffer.items()]))
        self._handle_calibration_stop()

    def get_prediction_field(self, block=True, timeout=None):
        try:
            freq = self.prediction_queue.get(block=block, timeout=timeout)
        except Empty:
            return None
        else:
            if freq:
                return self.stimulation_freqs.index(freq)
        return None
