import c3 from 'c3'
import global_settings from './settings'
import {redraw_chart_on_resize} from './charts'

initializeCC = (socket) ->
  chart = c3.generate
    bindto: 'classifier_certainty'
    data:
      columns: []
      labels: false
      colors:
        'YES': global_settings.choices.colors.yes
        'NO': global_settings.choices.colors.no
    axis:
      y:
        min: 0
        max: 1
  bind_to_socket(socket, chart)
  redraw_chart_on_resize(chart)


bind_to_socket = (socket, chart) ->
  socket.on 'classifier_certainty', (data) ->
    trimmed_columns = _trim_columns(data.columns)
    if trimmed_columns?
      chart.flow
        columns: trimmed_columns
        length: 0
        duration: 200
    else
      chart.load
        unload: true
        columns: data.columns
    chart.ygrids [value: data.threshold, text: 'threshold']


_trim_columns = (columns) ->
  previous_values_per_label = $('classifier_certainty').data('values_per_label')
  if not previous_values_per_label?
    previous_values_per_label = {}

  values_per_label = _columns_to_map(columns)
  $('classifier_certainty').data('values_per_label', values_per_label)

  new_values_per_label = {}
  for label of values_per_label
    if not previous_values_per_label[label]?
      return undefined
    previous_values = previous_values_per_label[label]
    previous_length = previous_values.length
    current_values = values_per_label[label]
    current_length = current_values.length
    if previous_length > current_length
      return undefined
    else if previous_length == 0
      new_values_per_label[label] = current_values
    else
      for v, i in current_values[...previous_length]
        previous_v = previous_values[i]
        if v != previous_v
          return undefined
      new_values_per_label[label] = current_values[previous_length..]

  return _map_to_columns(new_values_per_label)


_columns_to_map = (columns) ->
  map = {}
  for column in columns
    label = column[0]
    map[label] = column[1..]
  return map


_map_to_columns = (map) ->
  columns = []
  for key of map
    entry = [key]
    entry = entry.concat(map[key])
    columns.push(entry)
  return columns

export default initializeCC
