import c3 from 'c3'
import 'c3.css'
import EEGData from './eeg_data'
import global_settings from './settings'
import {redraw_chart_on_resize} from './charts'

REFRESH_RATE = 25

class EEGChart
  constructor: (@element, socket) ->
    @duration = parseInt($(@element).data('duration') ? 20)
    @duration = parseInt(20)
    @delay = parseFloat($(@element).data('delay') ? 0.5)
    @eeg_data = new EEGData(socket, @duration, @delay)
    @_chart = null
    @_run = true
    @_last_update =
    requestAnimationFrame(@_update_chart)
    $(@element).on 'remove', @destroy

  _update_chart: =>
    if @_run
      requestAnimationFrame(@_update_chart)
    if Date.now() < @_last_update + 1000 / REFRESH_RATE
      return
    data = @eeg_data.get_current_samples()
    if not data?
      return
    @_last_update = Date.now()
    delete data.timestamp
    if @_chart is null
      @_init_chart(data)
    else
      @_chart.load
        json: data
        redraw:
          withUpdateOrgXDomain: true,
          withUpdateXDomain: true,
          withLegend: false,
          withDimension: false,
          withUpdateAxis: false,

  _init_chart: (data) ->
    ticks = {}
    sampling_rate = @eeg_data.sampling_rate
    amplitude = @eeg_data.amplitude
    channel_names = @eeg_data.channel_names
    for c,i in @eeg_data.channel_names
      ticks[i * amplitude] = c
    @_chart = c3.generate
      bindto: @element
      data:
        json: data
        colors: global_settings.electrodes.colors
      point:
        show: false
      transition:
        duration: 0
      grid:
        x:
          show: false
          lines: ({value: i * sampling_rate, text: "", position: 'start', class: "dashed"} for i in [0..@duration])
        y:
          show: true
      tooltip:
        show: false
      padding:
        bottom: 30
      axis:
        x:
          tick:
            values: (i * sampling_rate for i in [0..@duration])
            format: (d) =>
              Math.abs(-@duration + d / sampling_rate).toFixed 0
          label:
            text: "Seconds ago"
            position: 'outer-center'
        y:
          min: channel_names.length * amplitude
          max: -amplitude / 2
          label:
            text: "Channel"
            position: 'outer-middle'
          padding:
            top: Math.abs(amplitude) / 2
            bottom: Math.abs(amplitude) / 2
          tick:
            values: (amplitude * i for i in [0...channel_names.length])
            format: (d) ->
              return ticks[d] ? ""
    @_chart.internal.redrawTitle = null
    redraw_chart_on_resize(@_chart)

  destroy: =>
    @_run = false

  @initialize: (socket) =>
    $('eeg_chart').each ->
      $(this).data('chart', new EEGChart(this, socket))


export default EEGChart
