import SignalReceiver from './signal_receiver'

class EEGData
  constructor: (socket, @max_length_in_s, @delay) ->
    @_receiver = new SignalReceiver(socket, 100000)
    @_last_update = Date.now()
    @_prev_chunk_end = null
    @_wait = false


  _push_samples: (channel_data) ->
    for name,data of channel_data when @channel_data[name]
      @channel_data[name] = @channel_data[name].concat(data)
      if @channel_data[name].length > @_max_length
        @channel_data[name] = @channel_data[name][@channel_data[name].length - @_max_length..]


  _init_buffers: ()->
    @channel_names = @_receiver.channel_names.filter (n) -> n != 'timestamp'
    @sampling_rate = @_receiver.sampling_rate
    @amplitude = @_receiver.max_amplitude
    @channel_data = {}
    @_max_length = Math.ceil(@max_length_in_s * @sampling_rate)
    for name in @channel_names
      @channel_data[name] = new Array(@_max_length)
      @channel_data[name].fill(null)
    @_prev_chunk_end = @_receiver.first_sample_timestamp - @delay

  get_current_samples: =>
    now = Date.now()
    update_duration = (now - @_last_update) / 1000.0
    @_last_update = now
    if @_prev_chunk_end is null
      if @_receiver.first_sample_timestamp?
        @_init_buffers()
    else
      delay = @delay * @_receiver.sampling_rate
      if @_receiver.length > delay * 2
        chunk_len = @_receiver.length - delay
      else if not @_wait or @_receiver.length > delay
        chunk_len = Math.round(Math.min(@_receiver.sampling_rate * update_duration, @_receiver.length))
      else
        chunk_len = 0
      samples = @_receiver._pop_samples(chunk_len)
      if samples?.timestamp?.length
        @_wait = false
        @_push_samples(samples)
        return @channel_data
      else
        @_wait = true
    return null

export default EEGData
