import c3 from 'c3'
import global_settings from './settings'
import {redraw_chart_on_resize} from './charts'

settings =
  colors: global_settings.classification.colors
  electrodes: electrodes = []
  amplitude: -4
  channels: _.flatten("#{electrode} #{type}" for electrode in electrodes for type in ['tgt', 'ntgt'])


initializeERP = (socket) ->
  chart = get_chart()
  redraw_chart_on_resize(chart)
  bind_to_socket(socket, chart)
  _update_counters(chart, 0, 0, false)

get_chart = (analysis_channels) ->
  if analysis_channels?
    settings.electrodes = analysis_channels
    settings.channels = _.flatten("#{electrode} #{type}" for electrode in electrodes for type in ['tgt', 'ntgt'])

  amplitude = settings.amplitude
  electrodes = settings.electrodes
  ticks = {}
  x_axis_values = [-0.2, 0, 0.3, 0.9]
  for c,i in electrodes
    ticks[i * amplitude] = c
  chart = c3.generate
    bindto: 'erp_chart'
    data:
      columns: [
        [_.last(settings.channels), -20]
        ['x_data', -0.2]
      ]
      labels: false
      x: 'x_data'
      colors: _.fromPairs(
        [channel, if /ntgt$/.test channel then settings.colors.nontarget else settings.colors.target
        ] for channel in settings.channels)
    point:
      show: false
    legend:
      show: false
    tooltip:
      show: false
    axis:
      y:
        min: (electrodes.length - 0.5) * amplitude
        max: -amplitude / 2
        tick:
          values: (amplitude * i for i in [0...electrodes.length])
          format: (d) -> ticks[d] ? ""
        label:
          text: "Channel"
          position: 'outer-middle'
      x:
        min: -0.2
        max: 1
        tick:
          values: x_axis_values
          format: (d) -> d * 1000
        label:
          text: "milliseconds since stimulus"
          position: 'outer-center'
        padding:
          left: 0.05

    grid:
      x:
        lines: [
          value: 0
          class: "zero"
        ]
  return chart


bind_to_socket = (socket, chart) ->
  socket.on 'erp', (data) ->
    if settings.channels.length == 0
      chart = get_chart(data.channels)
      socket.off 'erp'
      bind_to_socket(socket, chart)
    _update_counters(chart, data.targets_count, data.nontargets_count, data.calibration)
    _update_chart(chart, data.columns)


_update_counters = (chart, targets_count, nontargets_count, calibration) ->
  if calibration
    labels =
      target: 'TGT'
      nontarget: 'NTGT'
    colors =
      target: global_settings.classification.colors.target
      nontarget: global_settings.classification.colors.nontarget
  else
    labels =
      target: '"YES"'
      nontarget: '"NO"'
    colors =
      target: global_settings.choices.colors.yes
      nontarget: global_settings.choices.colors.no
  parent = $(chart.element).parent()
  parent.find('.target')
    .text("#{labels.target} = #{targets_count}")
    .css color: colors.target
  parent.find('.non-target')
    .text("#{labels.nontarget} = #{nontargets_count}")
    .css color: colors.nontarget

_update_chart = (chart, columns) ->
  if columns.length
    chart.load
      columns: columns
  else
    chart.load
      columns: ([label] for label in  settings.channels)

export default initializeERP
