class SignalReceiver
  #Signal buffering

  constructor: (@socket, @max_length)->
    @socket.on 'raw_signal_description', @_receive_signal_description
    @socket.on 'raw_signal', @_receive_raw_packet
    @_clear()

  _clear: =>
    @channel_names = null
    @_channel_data = {}
    @first_sample_timestamp = null
    @length = 0

  buffer_length_in_s: ->
    return @length / @sampling_rate

  _receive_signal_description: (data) =>
    @sampling_rate = data.sampling_rate
    @channel_names = data.channel_names
    @max_amplitude = data.max_amplitude

  _receive_raw_packet: (packet) =>
    if not @channel_names?
      return
    data = new Float32Array(packet)
    sample_count = data.length / @channel_names.length
    for name,i in @channel_names
      @_append_data(name, data.subarray(i * sample_count, (i + 1) * sample_count))
    if @first_sample_timestamp is null
      @first_sample_timestamp = @_channel_data.timestamp[0][0]
    @length += sample_count
    if @length > @max_length
      @_pop_samples(@length - @max_length)


  _append_data: (name, buffer)->
    @_channel_data[name] ?= []
    @_channel_data[name].push(buffer)

  pop_samples: (max_age) ->
    if not @channel_names?
      return null
    sample_count = @_find_nr_of_sample_to_return(max_age)
    return @_pop_samples(sample_count)

  _find_nr_of_sample_to_return: (max_age) ->
    if not @_channel_data?.timestamp?
      return 0
    sample_count = 0
    chunk_nr = 0
    while chunk_nr < @_channel_data.timestamp.length
      timestamp_chunk = @_channel_data.timestamp[chunk_nr]
      if timestamp_chunk[timestamp_chunk.length - 1] < max_age
        chunk_nr += 1
        sample_count += timestamp_chunk.length
      else
        sample_count += timestamp_chunk.findIndex (v)->v >= max_age
        break

    return sample_count

  _pop_samples: (sample_count)->
    samples = {}
    for name in @channel_names
      samples[name] = @_pop_channel_samples(name, sample_count)
    @length -= sample_count
    return samples

  _pop_channel_samples: (name, sample_count) ->
    samples = new Float32Array(sample_count)
    cur_offset = 0
    channel_data = @_channel_data[name]
    while cur_offset < sample_count
      if channel_data[0].length + cur_offset <= sample_count
        samples.set(channel_data[0], cur_offset)
        cur_offset += channel_data[0].length
        channel_data.shift()
      else
        samples.set(channel_data[0].subarray(0, sample_count - cur_offset), cur_offset)
        channel_data[0] = channel_data[0].subarray(sample_count - cur_offset)
        cur_offset = sample_count
    return Array.from(samples)


export default SignalReceiver
