import {expect} from 'chai'
import SignalReceiver from 'signal_receiver'
create_generator = (channels, sampling_rate, amplitude) ->
    timestamp = 0
    generate_chunk = (chunk_length)->
      data = new Float32Array((channels + 1) * chunk_length)
      for i in [0...chunk_length]
        data[i] = timestamp
        timestamp += 1.0 / sampling_rate
      for channel in [1..channels]
        for i in [0...chunk_length]
          data[channel * chunk_length + i] = (channel + Math.random() - 0.5) * amplitude
      return data
    return generate_chunk

describe "SignalReceiver", ->
  describe "signal_generator", ->
    it "should generate proper data chunks", ->
      signal_generator = create_generator(2, 128, 5)
      chunk = signal_generator(2)
      expect(chunk).to.have.lengthOf(6)
      expect(chunk[0]).to.equal(0)
      expect(chunk[1]).to.equal(1.0 / 128)
      expect(chunk[2]).to.be.closeTo(5, 5)
      expect(chunk[3]).to.be.closeTo(5, 5)
      expect(chunk[4]).to.be.closeTo(10, 5)
      expect(chunk[5]).to.be.closeTo(10, 5)
  beforeEach ->
    @socket = {on: (name, func)-> this[name] = func}
    @signal_desc =
      channel_names: ['timestamp', 'C1', 'C2']
      sampling_rate: 128
      max_amplitude: 5
    @signal_generator = create_generator(
      @signal_desc.channel_names.length - 1
      @signal_desc.sampling_rate
      @signal_desc.max_amplitude)
    @receiver = new SignalReceiver(@socket, 5)
    @socket.raw_signal_description @signal_desc
  it "should receive samples", ->
    r = this.receiver
    signal_generator = @signal_generator
    @socket.raw_signal(signal_generator(2))
    @socket.raw_signal(signal_generator(3))
    expect(r).to.have.lengthOf(5)
    data = r.pop_samples(1)
    expect(data).to.have.all.keys('timestamp','C1','C2')
    expect(data.timestamp).to.have.length(5)
    expect(r).to.have.lengthOf(0)
    @socket.raw_signal(signal_generator(5))
    @socket.raw_signal(signal_generator(5))
    expect(r).to.have.lengthOf(5)
  it "should return chunks of  proper length", ->
    signal_generator = @signal_generator
    @socket.raw_signal(signal_generator(2))
    @socket.raw_signal(signal_generator(2))
    @socket.raw_signal(signal_generator(2))
    expect(@receiver).to.have.lengthOf(5)
    data = @receiver.pop_samples(3.0/@signal_desc.sampling_rate)
    expect(data.timestamp).to.have.length(2)
    expect(@receiver).to.have.lengthOf(3)
    data = @receiver.pop_samples(3.0/@signal_desc.sampling_rate)
    expect(data.timestamp).to.have.length(0)
    data = @receiver.pop_samples(5.0/@signal_desc.sampling_rate)
    expect(data.timestamp).to.have.length(2)
    data = @receiver.pop_samples(6.0/@signal_desc.sampling_rate)
    expect(data.timestamp).to.have.length(1)




