# -*- coding: utf-8 -*-
# Copyright (c) 2016-2018 Braintech Sp. z o.o. [Ltd.] <http://www.braintech.pl>
# All rights reserved.
import numpy as np

from bci_panel import bci_panels
from braintech.obci.signal_processing.signal.data_generic_write_proxy import SamplePacket
from .ssvep_analysis import SSVEPAnalysis
from ..utils.visualization import SsvepVisualizationDispatcher


class SSVEPAnalysisVisualisation(SSVEPAnalysis):
    def __init__(self, *args, bci_panel=None,
                 bci_panel_class=bci_panels.DummyBCIPanelProcess, **kwargs):
        super().__init__(*args, **kwargs)
        if bci_panel is None:
            self.bci_panel = SsvepVisualizationDispatcher(
                channel_names=(self.channels_for_montage
                               + self.channels_for_classification),
                channels_after_montage=self.channels_for_classification,
                sampling_rate=self.sample_rate,
                stimulation_freqs=self.stimulation_freqs,
                bci_panel_class=bci_panel_class,
            )
        else:
            self.bci_panel = bci_panel

    def load_wisdom(self, path):
        super().load_wisdom(path)
        if self.calibration_result.is_valid():
            self.bci_panel.handle_cv_result(self.calibration_result)

    def _handle_calibration_done(self):
        super()._handle_calibration_done()
        self.bci_panel.handle_cv_result(self.calibration_result)

    def handle_sample_packet(self, packet: SamplePacket):
        self.bci_panel.handle_raw_signal(packet)
        super().handle_sample_packet(packet)
        montaged_packet = SamplePacket(self.ssvep_montage(packet.samples), packet.ts)
        self.bci_panel.handle_montaged_signal(montaged_packet)

    def _add_features_for_learning(self, features):
        super()._add_features_for_learning(features)

        x_tick_labels = []
        for f in self.stimulation_freqs:
            x_tick_labels.append(str(f))

        x_tick_locations = np.arange(0, len(features))

        self.bci_panel.handle_features(features=features, calibration_type=self.calibration_type,
                                       x_ticks=[x_tick_locations, x_tick_labels])

    def close(self):
        super().close()
        self.bci_panel.close()
