# -*- coding: utf-8 -*-
# Copyright (c) 2016-2018 Braintech Sp. z o.o. [Ltd.] <http://www.braintech.pl>
# All rights reserved.

import numpy as np
import pylab as pb

from bci_ssvep.logic.ssvep_frequency_finder import SSVEPFrequencyFinder


def prepare_data(stim_freq=30, sfreq=512, length=2, noise_level=10, amplitude=4, phase=1.1):
    n_samples = sfreq * length
    signal = np.zeros(n_samples)
    timeline = np.linspace(0, length - 1 / sfreq, n_samples)
    signal += amplitude * np.sin(2 * np.pi * stim_freq * timeline + phase)
    signal += np.random.normal(0, noise_level, n_samples)
    return timeline, signal[:, None]


if __name__ == '__main__':
    s = SSVEPFrequencyFinder()

    freqs = np.linspace(25, 40, 1000)

    predictions = []
    for f in freqs:

        timeline, signal1 = prepare_data(stim_freq=f)
        timeline, signal2 = prepare_data(stim_freq=f, amplitude=3)
        timeline, signal3 = prepare_data(stim_freq=32, amplitude=1)

        signal = np.concatenate((signal1 + signal3, signal2 + signal3, signal3), axis=1)

        prediction = s.predict(signal)
        predictions.append(prediction)

    predictions = np.array(predictions)

    for i, f in enumerate(s.stimulation_freqs):
        pb.plot(freqs, predictions[:, i], label=str(f))
    pb.legend()
    pb.show()
