# Copyright (c) 2016-2018 Braintech Sp. z o.o. [Ltd.] <http://www.braintech.pl>
# All rights reserved.

from bci_ssvep.logic.ssvep_calibrator import CalibrationStartedDebugMsg
from braintech.obci.core.broker.message_handler_mixin import subscribe_message_handler
from braintech.obci.experiment.peers.drivers.amplifiers.amplifier_peer import AmplifierPeer
from obci_p300.common.random_amp_with_sine.random_amplifier_peer_with_sine_generator import \
    BaseRandomAmplifierWithSinePeer

__all__ = ('SSVEPRandomAmplifierPeer',)


class BaseSSVEPRandomAmplifierPeer(BaseRandomAmplifierWithSinePeer):
    """Random amplifier - sends random data (uniform [0;1)) over all channels."""

    def __init__(self, *args, **kwargs):
        self._autocalibrate_ssvep = False
        super(BaseSSVEPRandomAmplifierPeer, self).__init__(*args, **kwargs)

    def ssvep_autocalibration_check_handle(self, state):
        self._autocalibrate_ssvep = state > 0

    def _create_window(self):
        super()._create_window()
        self._window.autocalibrate_ssvep_changed.connect(self.ssvep_autocalibration_check_handle)

    @subscribe_message_handler(CalibrationStartedDebugMsg)
    async def handle_calibration_debug_message(self, msg: CalibrationStartedDebugMsg):
        if self._autocalibrate_ssvep:
            freq = float(msg.frequency)
            if self._window:
                self._window.update_ssvep_freq.emit(freq)
            else:
                self.amplifier_frequency_select_handle(freq)


class SSVEPRandomAmplifierPeer(BaseSSVEPRandomAmplifierPeer, AmplifierPeer):
    pass
