# -*- coding: utf-8 -*-
# Copyright (c) 2016-2018 Braintech Sp. z o.o. [Ltd.] <http://www.braintech.pl>
# All rights reserved.
import logging

import numpy as np

from braintech.drivers.blinker.blinker import WrongDeviceNameException, get_blinker
from braintech.drivers.blinker.fat_blinker import FatBlinker  # noqa

logger = logging.getLogger(__name__)


def set_fat_blinker_60_fields(blinker):
    blinker.set_number_of_fields(60)
    for i in range(10):
        blinker.define_field(i, list(range(i * 8, (i + 1) * 8 - 4)))

    for i in range(10, 20):
        channels = list(range((i - 10) * 8 + 5, (i - 10 + 1) * 8 + 5 - 6))
        channels.extend(list(range((i - 10) * 8 + 80, (i - 10 + 1) * 8 + 80 - 6)))
        blinker.define_field(i, channels)

    for i in range(20, 30):
        channels = list(range((i - 20) * 8 + 83, (i - 20 + 1) * 8 + 83 - 4))
        blinker.define_field(i, channels)

    for i in range(30, 40):
        channels = list(range((i - 30) * 8 + 161, (i - 30 + 1) * 8 + 161 - 4))
        blinker.define_field(i, channels)

    for i in range(40, 50):
        channels = list(range((i - 40) * 8 + 166, (i - 40 + 1) * 8 + 166 - 7))
        channels.extend(list(range((i - 40) * 8 + 240, (i - 40 + 1) * 8 + 240 - 5)))
        blinker.define_field(i, channels)

    for i in range(50, 60):
        channels = list(range((i - 50) * 8 + 244, (i - 50 + 1) * 8 + 244 - 5))
        blinker.define_field(i, channels)


def set_fat_blinker_50_fields(blinker):
    blinker.set_number_of_fields(50)
    for i in range(10):
        blinker.define_field(i, list(range(i * 8, (i + 1) * 8 - 4)))

    for i in range(10, 20):
        channels = list(range((i - 10) * 8 + 6, (i - 10 + 1) * 8 + 6 - 7))
        channels.extend(list(range((i - 10) * 8 + 80, (i - 10 + 1) * 8 + 80 - 5)))
        blinker.define_field(i, channels)

    for i in range(20, 30):
        channels = list(range((i - 20) * 8 + 85, (i - 20 + 1) * 8 + 85 - 6))
        channels.extend(list(range((i - 20) * 8 + 160, (i - 20 + 1) * 8 + 160 - 6)))
        blinker.define_field(i, channels)

    for i in range(30, 40):
        channels = list(range((i - 30) * 8 + 164, (i - 30 + 1) * 8 + 164 - 5))
        channels.extend(list(range((i - 30) * 8 + 240, (i - 30 + 1) * 8 + 240 - 7)))
        blinker.define_field(i, channels)

    for i in range(40, 50):
        channels = list(range((i - 40) * 8 + 243, (i - 40 + 1) * 8 + 243 - 4))
        blinker.define_field(i, channels)


def set_fat_blinker_40_fields_center_status_noncontionious(blinker):
    blinker.set_number_of_fields(41)

    status_channels = []
    for i in range(20, 30):
        status_channels.extend(list(range((i - 20) * 8 + 85, (i - 20 + 1) * 8 + 85 - 6)))
        status_channels.extend(list(range((i - 20) * 8 + 160, (i - 20 + 1) * 8 + 160 - 6)))
    blinker.define_field(40, status_channels)

    for i in range(10):
        blinker.define_field(i, list(range(i * 8, (i + 1) * 8 - 4)))

    for i in range(10, 20):
        channels = list(range((i - 10) * 8 + 6, (i - 10 + 1) * 8 + 6 - 7))
        channels.extend(list(range((i - 10) * 8 + 80, (i - 10 + 1) * 8 + 80 - 5)))
        blinker.define_field(i, channels)

    for i in range(30, 40):
        channels = list(range((i - 30) * 8 + 164, (i - 30 + 1) * 8 + 164 - 5))
        channels.extend(list(range((i - 30) * 8 + 240, (i - 30 + 1) * 8 + 240 - 7)))
        blinker.define_field(i - 10, channels)

    for i in range(40, 50):
        channels = list(range((i - 40) * 8 + 243, (i - 40 + 1) * 8 + 243 - 4))
        blinker.define_field(i - 10, channels)


def set_fat_blinker_40_fields_center_status_contionious(blinker):
    blinker.set_number_of_fields(41)

    status_channels = []
    for i in range(20, 30):
        status_channels.extend(list(range((i - 20) * 8 + 85, (i - 20 + 1) * 8 + 85 - 6)))
        status_channels.extend(list(range((i - 20) * 8 + 160, (i - 20 + 1) * 8 + 160 - 6)))
        status_channels.append((i - 20) * 8 + 87)
        status_channels.append((i - 20) * 8 + 167)
    blinker.define_field(40, status_channels)

    for i in range(10):
        blinker.define_field(i, list(range(i * 8, (i + 1) * 8 - 4)))

    for i in range(10, 20):
        channels = list(range((i - 10) * 8 + 6, (i - 10 + 1) * 8 + 6 - 7))
        channels.extend(list(range((i - 10) * 8 + 80, (i - 10 + 1) * 8 + 80 - 5)))
        blinker.define_field(i, channels)

    for i in range(30, 40):
        channels = list(range((i - 30) * 8 + 164, (i - 30 + 1) * 8 + 164 - 5))
        channels.extend(list(range((i - 30) * 8 + 240, (i - 30 + 1) * 8 + 240 - 7)))
        blinker.define_field(i - 10, channels)

    for i in range(40, 50):
        channels = list(range((i - 40) * 8 + 243, (i - 40 + 1) * 8 + 243 - 4))
        blinker.define_field(i - 10, channels)


def set_fat_blinker_40_fields_top_status_noncontinious(blinker):
    blinker.set_number_of_fields(41)

    status_channels = []
    for i in range(10):
        status_channels.extend(list(range(i * 8, (i + 1) * 8 - 4)))

    blinker.define_field(40, status_channels)

    for i in range(10, 20):
        channels = list(range((i - 10) * 8 + 6, (i - 10 + 1) * 8 + 6 - 7))
        channels.extend(list(range((i - 10) * 8 + 80, (i - 10 + 1) * 8 + 80 - 5)))
        blinker.define_field(i - 10, channels)

    for i in range(20, 30):
        channels = list(range((i - 20) * 8 + 85, (i - 20 + 1) * 8 + 85 - 6))
        channels.extend(list(range((i - 20) * 8 + 160, (i - 20 + 1) * 8 + 160 - 6)))
        blinker.define_field(i - 10, channels)

    for i in range(30, 40):
        channels = list(range((i - 30) * 8 + 164, (i - 30 + 1) * 8 + 164 - 5))
        channels.extend(list(range((i - 30) * 8 + 240, (i - 30 + 1) * 8 + 240 - 7)))
        blinker.define_field(i - 10, channels)

    for i in range(40, 50):
        channels = list(range((i - 40) * 8 + 243, (i - 40 + 1) * 8 + 243 - 4))
        blinker.define_field(i - 10, channels)


def set_fat_blinker_40_fields_top_status_continious(blinker):
    blinker.set_number_of_fields(41)

    status_channels = []
    for i in range(10):
        status_channels.extend(list(range(i * 8, (i + 1) * 8 - 2)))
        status_channels.append(8 * (i + 1) - 1)
    blinker.define_field(40, status_channels)

    for i in range(10, 20):
        channels = list(range((i - 10) * 8 + 81, (i - 10 + 1) * 8 + 81 - 4))
        blinker.define_field(i - 10, channels)

    for i in range(20, 30):
        channels = list(range((i - 20) * 8 + 86, (i - 20 + 1) * 8 + 86 - 7))
        channels.extend(list(range((i - 20) * 8 + 160, (i - 20 + 1) * 8 + 160 - 5)))
        blinker.define_field(i - 10, channels)

    for i in range(30, 40):
        channels = list(range((i - 30) * 8 + 164, (i - 30 + 1) * 8 + 164 - 5))
        channels.extend(list(range((i - 30) * 8 + 240, (i - 30 + 1) * 8 + 240 - 7)))
        blinker.define_field(i - 10, channels)

    for i in range(40, 50):
        channels = list(range((i - 40) * 8 + 242, (i - 40 + 1) * 8 + 242 - 4))
        blinker.define_field(i - 10, channels)


class SSVEPBlinkerController:
    def __init__(self, stim_freqs, status_bar_light_indexes, fields_ids_light_indexes, phase_randomistation=True,
                 field_setup_func=None):

        try:
            self.blinker = get_blinker('auto')  # type:FatBlinker
        except WrongDeviceNameException:
            self.blinker = get_blinker('dummy_fat_blinker', preview=True)  # type:FatBlinker
        self.blinker.set_intensity(1.0)
        self.stim_freqs = stim_freqs
        self.status_bar_light_indexes = status_bar_light_indexes
        self.fields_ids_light_indexes = fields_ids_light_indexes
        self.fullbright()
        self._phase_randomistation = phase_randomistation
        if field_setup_func is not None:
            field_setup_func(self.blinker)

    def update_stim_freqs(self, stim_freqs):
        assert len(stim_freqs) == len(self.stim_freqs)
        self.stim_freqs = stim_freqs[::]

    def start_stimulation(self):
        for field in range(self.channels_number):
            self.blinker.disable(field, 0)

        for field in self.status_bar_light_indexes:
            self.blinker.disable(field, 1)

        for freq, fields in zip(self.stim_freqs, self.fields_ids_light_indexes):
            if self._phase_randomistation:
                phase_offset_per_freq = np.random.uniform(0, 360)
            else:
                phase_offset_per_freq = 0
            for field in fields:
                self.blinker.enable(field, 1)
                self.blinker.set_rectangular_function(field, freq, duty_cycle=0.5, luminosity=1,
                                                      phase_offset=phase_offset_per_freq)
        self.blinker.start()

    def fullbright(self):
        self.blinker.stop()
        for field in range(self.channels_number):
            self.blinker.enable(field, 1)
            self.blinker.set_still(field, 1.0)
        self.blinker.start()

    @property
    def channels_number(self):
        if hasattr(self.blinker, 'fields_num'):
            logger.warning('The new property is working. '
                           'Please remove this transitional property.')
            return self.blinker.fields_num
        elif hasattr(self.blinker, 'CHANNELS_NUMBER'):
            return self.blinker.CHANNELS_NUMBER
        else:
            raise NotImplementedError

    def close(self):
        blinker_preview = getattr(self.blinker, '_preview', None)
        if blinker_preview is not None:
            blinker_preview.proc.terminate()


if __name__ == '__main__':
    s = SSVEPBlinkerController([30, 35], list(range(10)), [[21, 22], [27, 28]])
    import IPython

    IPython.embed()
