# Copyright (c) 2016-2018 Braintech Sp. z o.o. [Ltd.] <http://www.braintech.pl>
# All rights reserved
import socket
import sys

import PySide2
from PySide2 import QtCore, QtWidgets, QtWebChannel

# noinspection PyUnresolvedReferences
import obci_demo.gui.bci_demo_resources_rc  # noqa
from obci_demo.gui.bci_controller import logger, BCIDemoController
from obci_demo.gui.bci_frontend import OBCIDemoExperimentWithFrontend, P300BCIFrontend
from obci_demo.gui.web_view import WebView


class BCIDemoWindow(QtWidgets.QMainWindow):

    def __init__(self, bci_demo_controller, title):
        QtWidgets.QMainWindow.__init__(self)
        self.setWindowTitle(title)
        self.view = WebView(self)
        self.page = self.view.page()
        channel = QtWebChannel.QWebChannel(self)
        self.page.setWebChannel(channel)
        channel.registerObject("bci_demo", bci_demo_controller)

        self.setCentralWidget(self.view)
        self.view.loadFinished.connect(self.showFullScreen)

    def load(self, url):
        logger.info("Running from url: %s", url)
        self.view.load(QtCore.QUrl(url))

    closed = QtCore.Signal()

    def closeEvent(self, event: PySide2.QtGui.QCloseEvent):
        self.closed.emit()


class P300BCIDemo:
    PREFIX = 'demo'
    WINDOW_TITLE = 'P300 Demo'

    def _is_devserver(self):
        sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        sock.settimeout(.1)
        result = sock.connect_ex(('127.0.0.1', 8080))
        sock.close()
        return result == 0

    def __init__(self, config):
        self._config = config
        self.frontend = self.create_frontend()
        self.experiment = self.create_experiment()
        self.experiment.bci_panel.set_prefix(self.PREFIX)
        self.controller = self.create_controller()
        self.main_window = self.create_window()
        self.main_window.closed.connect(self.controller.close)

    def create_frontend(self):
        return P300BCIFrontend(self._config['p300']['field_labels'])

    def create_experiment(self):
        experiment = OBCIDemoExperimentWithFrontend(self._config, self.frontend)
        return experiment

    def create_controller(self):
        return BCIDemoController(self._config, self.frontend, self.experiment)

    def create_window(self):
        return BCIDemoWindow(self.controller, self.WINDOW_TITLE)

    def start(self):
        if self._is_devserver():
            url = "http://127.0.0.1:8080"
            frontend_url = "http://127.0.0.1:8080/frontend.html"
        else:
            url = "qrc:/%s/index.html" % self.PREFIX
            frontend_url = "qrc:/%s/frontend.html" % self.PREFIX
        self._url = url
        self._frontend_url = frontend_url
        self.main_window.load(url)
        self.frontend.load(frontend_url)


def run(config, DemoClass=P300BCIDemo):
    app = QtWidgets.QApplication(sys.argv + ['--disable-web-security'])
    demo_app = DemoClass(config)
    demo_app.start()
    app.exec_()


if __name__ == '__main__':
    from obci_demo.entrypoint import get_argument_parser

    parser = get_argument_parser()
    run(parser.parse_known_args(sys.argv))
