import 'jqueryui'
import {QWebChannel} from 'qwebchannel'
import BCIPanel from 'bci_panel'
import style from './bci_app.scss'
import BCIConfig from './bci_config'
import MockTransport from './mock_transport'

class AppStatus
  @STOPPED = 'stopped'
  @READY = 'ready'
  @CALIBRATING = 'calibrating'
  @COMMUNICATING = 'communicating'
  @CALIBRATED = 'calibrated'


class BCIApplication
  ConfigClass: BCIConfig
  constructor: (element, panel_address)->
    @_el = $(element)
    @_status = @_el.find('#status_text')
    @_buttons = @_el.find('#start_calibration,#start_bci,#stop_amplifier')
    if not window.qt? and window.location.search.search('force_qt') > 0
      window.qt =
        webChannelTransport: new MockTransport()
    if window.qt?
      @_web_channel = new QWebChannel qt.webChannelTransport, @_init_webchannel
    else
      @_el.find('#config,.control-buttons').hide()
      @_bci_panel = @_init_panel(window.location.host)
    @_configure_amplifier = @_el.find("#stop_amplifier")
    @_calibration = @_el.find("#start_calibration")
    @_bci = @_el.find("#start_bci")
    @_configure_amplifier.on 'click', (evt) =>
      $(event.target).prop('disabled', true)
      if @_bci_demo.status != AppStatus.STOPPED
        @_bci_demo.stop_amplifier()
      else
        @_config.show()
    @_calibration.on 'click', @_button_clicked
    @_bci.on 'click', @_button_clicked

  _init_form: (amplifiers)->
    @_config = new @ConfigClass(@_el.find('form'), amplifiers, @_start_amplifier, @_stop_amplifier)

  _update_buttons: (status) =>
    if status == AppStatus.CALIBRATING
      @_calibration.html("Finish calibration")
    else
      @_calibration.html("Start calibration")
    if status == AppStatus.COMMUNICATING
      @_bci.html("Finish BCI session")
    else
      @_bci.html("Start BCI session")
    @_configure_amplifier.prop('disabled', status == AppStatus.STOPPED)

    @_calibration.prop('disabled', status in [AppStatus.STOPPED, AppStatus.COMMUNICATING])
    @_bci.prop('disabled', status in [AppStatus.STOPPED, AppStatus.CALIBRATING, AppStatus.READY])
    @_status.html(status)

    if status == AppStatus.STOPPED
      @_config.show()

  _init_panel: (address)->
    @_el.find('.grid-stack').removeClass('grid-stack')
    @_el.find('.grid-stack-item').removeClass('grid-stack-item').addClass('graph-container').each ->
      $(this).html($(this).find('.grid-stack-item-content').html())
    @_bci_panel = new BCIPanel(@_el.find('#panel'), address, )
  _init_webchannel: (channel) =>
    @_bci_demo = channel.objects.bci_demo
    @_buttons.on 'click', @_button_clicked
    @_bci_demo.status_changed.connect @_status_changed
    bci_panel_addresses = @_bci_demo.settings.bci_panel_addresses

    if bci_panel_addresses.length > 0
      address = bci_panel_addresses[bci_panel_addresses.length - 1]
      @_el.find("#title").html(@_bci_demo.settings.title)
        .after("<p>To view panel on another device open <a href=\"http://#{address}\">http://#{address}</a>
          in browser.</p>")
    @_init_panel(bci_panel_addresses[0])
    @_init_form(@_bci_demo.amplifiers)
    @_status_changed(@_bci_demo.status)



  _start_amplifier: (config) =>
    result = $.Deferred()
    @_bci_panel.reset()
    @_bci_demo.start_amplifier(config)
    @_el.one 'status_changed', =>
      if @_bci_demo.status == AppStatus.READY
        result.resolve()
      else
        result.reject()
    result.resolve()
    return result.promise()

  _stop_amplifier: (config) =>
    result = $.Deferred()
    @_bci_demo.stop_amplifier(config)
    @_el.one 'status_changed', =>
      if @_bci_demo.status == AppStatus.STOPPED
        result.resolve()
      else
        result.reject()
    return result.promise()

  _status_changed: (new_status) =>
    console.log "new status " + @_bci_demo.status
    @_update_buttons(@_bci_demo.status)
    @_el.trigger('status_changed')

  _button_clicked: (event) =>
    $(event.target).prop('disabled', true)
    if @_bci_demo.status in [AppStatus.CALIBRATING, AppStatus.COMMUNICATING]
      @_bci_demo.stop()
    else
      but_name = $(event.target).attr('id')
      @_bci_demo[but_name]()


export default BCIApplication
