class P300Blinker
  CLASSES = ['fullblink', 'halfblink', 'selected']

  constructor: (p300_display) ->
    @p300_display = p300_display
    @fields = p300_display.find('.p300_field')
    @fields.each ->
      $(this).html("""<div class="progress"></div><div class="threshold"></div>
         <img class="blink_image"/><div class="text">#{$(this).html()}</div>""")
    @instruction = p300_display.find("#instruction")
    $('body').on 'keydown', (evt) =>
      @frontend.key_pressed_emit(evt.key)
    @p300_display.find('#full_screen').on 'click', =>
      @frontend.toggle_fullscreen(not @frontend.fullscreen)
    @p300_display.find("#stop").on 'click', =>
      @frontend.stop()
    @_handlers = {}


  setChannel: (@frontend) ->
    @frontend.field_config_changed.connect(@_setFieldConfig)
    @frontend.mode_changed.connect(@_setMode)
    @frontend.instruction_changed.connect(@_setInstruction)
    @frontend.fullscreen_changed.connect =>
      @p300_display.find('#full_screen i').html(if @frontend.fullscreen then 'fullscreen_exit' else 'fullscreen')
    @frontend._event.connect(@_call_handlers)
    @_setFieldConfig()
    @_setMode()
    @_setInstruction()

  _call_handlers: (event_name, data) =>
    for h in @_handlers[event_name] ? []
      h(data)

  toggle: (event_name, data) =>
    @frontend.toggle(event_name, data)

  on: (event_name, handler) ->
    (@_handlers[event_name]?=[]).push(handler)

  off: (event_name, handler) ->
    (@_handlers[event_name] ? []).remove(handler)

  _setFieldConfig: (fields_config)=>
    fields_config = fields_config ? @frontend.field_config
    for field,i in @fields
      field_config = fields_config[i]
      state = field_config.state ? field_config
      $(field).attr(state: state)
      if field_config.label?
        $(field).find('.text').html(field_config.label)

    requestAnimationFrame =>
      @frontend.display_updated(Date.now() / 1000)


  _setMode: ()=>
    @p300_display.attr(class: @frontend.mode)

  _setInstruction: (text) =>
    @instruction.html(@frontend.instruction)

export default P300Blinker
