const path = require('path');
const webpack = require('webpack');
const HtmlWebpackPlugin = require('html-webpack-plugin');
const CleanWebpackPlugin = require('clean-webpack-plugin');
const MiniCssExtractPlugin = require("mini-css-extract-plugin");
const devMode = process.env.NODE_ENV !== 'production'
const CopyWebpackPlugin = require('copy-webpack-plugin')
const output_dir = path.resolve(__dirname, 'obci_demo/gui/dist')
const src_dir = path.resolve(__dirname, 'static')
const bci_panel_files = path.resolve(__dirname, '../bci-panel/static')
const node_modules = path.resolve(__dirname, '../node_modules')
module.exports = {
  context: src_dir,
  entry: {
    app: './p300_app.coffee',
    frontend: './p300_frontend.coffee',
  },
  output: {
    path: output_dir,
    filename: '[name].bundle.js',
  },
  resolve: {
    modules: [src_dir, node_modules],
    alias: {
      "gridstack.scss": "gridstack/src/gridstack.scss",
      'gridstack.jqueryUI': 'gridstack/dist/gridstack.jQueryUI.js',
      'jquery-ui': 'jquery-ui/ui',
      'c3.css': 'c3/c3.css',

    },
    extensions: ['.coffee', '.js']
  },
  devtool: 'cheap-module-eval-source-map',
  module: {
    rules: [
      {
        test: /\.coffee$/i,
        use: [{
          loader: 'coffee-loader',
          options: {
            transpile: {
              presets: ['env']
            }
          }
        }]
      },
      {
        test: /\.(sass|scss)$/i,
        use: [
          devMode ? 'style-loader' : MiniCssExtractPlugin.loader,
          'css-loader',
          {
            loader: 'postcss-loader', // Run post css actions
            options: {
              plugins: function () { // post css plugins, can be exported to postcss.config.js
                return [
                  require('autoprefixer')
                ];
              }
            },
          },
          'sass-loader',
        ]
      },
      {
        test: /\.css$/i,
        use: [
          devMode ? 'style-loader' : MiniCssExtractPlugin.loader,
          'css-loader',
          {
            loader: 'postcss-loader', // Run post css actions
            options: {
              plugins: function () { // post css plugins, can be exported to postcss.config.js
                return [
                  require('autoprefixer')
                ];
              }
            },
          }
        ]
      },
      {
        test: /test\.js$/,
        use: 'mocha-loader',
        exclude: /node_modules/,
      }, {
        test: /\.svg$/,
        loader: 'svg-inline-loader'
      },
      {
        test: /\.(jpg|png)$/,
        use: {
          loader: "url-loader",
          options: {
            limit: 25000,
          },
        },
      },
      {
        test: /\.(eot|woff|woff2|ttf)([\?]?.*)$/,
        loader: "file-loader"
      },
    ]
  },
  devServer: {
    contentBase: [bci_panel_files],
    // publicPath: '/bci_panel',
    // hot: true
  },
  plugins: [
    new CleanWebpackPlugin([output_dir]),
    new HtmlWebpackPlugin({
      title: 'BCI Demo application',
      template: path.resolve(src_dir, 'index.html'),
      excludeChunks: ['frontend']
    }),
    new HtmlWebpackPlugin({
      title: 'BCI frontend',
      template: path.resolve(src_dir, 'frontend.html'),
      filename: 'frontend.html',
      excludeChunks: ['app']
    }),
    new MiniCssExtractPlugin({
      // Options similar to the same options in webpackOptions.output
      // both options are optional
      filename: devMode ? '[name].css' : '[name].[hash].css',
      chunkFilename: devMode ? '[id].css' : '[id].[hash].css',
    }),
    new webpack.NamedModulesPlugin(),
    new webpack.ProvidePlugin({
      $: "jquery",
      jQuery: "jquery"
    }),
    new webpack.HotModuleReplacementPlugin(),
    new webpack.IgnorePlugin(/^\.\/locale$/, /moment$/),
    new CopyWebpackPlugin([{from: path.resolve(bci_panel_files, 'img'), to: 'img'}], {}),
  ],
  optimization: {
    splitChunks: {
      cacheGroups: {
        commons: {
          test: /[\\/]node_modules[\\/]/,
          name: 'vendor',
          chunks: 'all'
        }
      }
    }
  }
};
