# -*- coding: utf-8 -*-
# Copyright (c) 2016-2018 Braintech Sp. z o.o. [Ltd.] <http://www.braintech.pl>
# All rights reserved.

"""This module defines helper classes for messages."""
from braintech.obci.signal_processing.buffers.auto_blink_buffer import Blink
from braintech.obci.core.broker.messages.base import BaseMessage
from braintech.obci.core.broker.messages import Field


class BlinkMsg(BaseMessage):  # noqa
    __TYPE__ = 'BLINK_MESSAGE'
    index = Field(int)
    timestamp = Field(float)

    def get_blink(self):
        return Blink(self.timestamp, self.index)


class BlinkConverter:
    """:class:'BlinkConverter' converts tag to blink."""

    def __init__(self, index_lambda=lambda t: int(t['desc']['blink'])):
        """
        Convert single tag to blink, when tag`s name is identical tu 'rule' determined by index_lambda.

        :param index_lambda: expression - meaning that rule is determined by a function
        """
        self.index_lambda = index_lambda

    def get_message(self, tag, ts=None):
        """
        Convert tag to blink message.

        :return msg_types.BLINK_MESSAGE: message type for blinks
        :return blink_msg 'dict': contain 'timestamp' and 'index'
        """
        if ts is None:
            ts = tag['start_timestamp']
        return BlinkMsg(timestamp=ts, index=self.index_lambda(tag))
