# Copyright (c) 2016-2018 Braintech Sp. z o.o. [Ltd.] <http://www.braintech.pl>
# All rights reserved.
from PySide2 import QtWidgets
from PySide2.QtCore import Signal, QTimer

from .random_app_with_sine_ui import Ui_RandomAmpWithSineWindow


class RandomAmplifierWithSineAppWindow(QtWidgets.QMainWindow, Ui_RandomAmpWithSineWindow):
    update_last_blink = Signal(str)
    update_ssvep_freq = Signal(float)
    update_react_to = Signal(str)

    def __init__(self, frequency_set_handler, amp_set_handler, noise_std_handler, p300_amplitude_handler,
                 generate_p300_handler, p300_autocalibration_handle,
                 blink_reaction_text_updated):
        super().__init__()
        self.setupUi(self)
        self.freq_box.valueChanged.connect(frequency_set_handler)
        self.sine_lvl.valueChanged.connect(amp_set_handler)
        self.noise_std.valueChanged.connect(noise_std_handler)
        self.p300_amplitude.valueChanged.connect(p300_amplitude_handler)
        self.p300_generatio_button.pressed.connect(generate_p300_handler)
        self.autocalibrate_ssvep_changed = self.autocalibrate_ssvep_box.stateChanged
        self.autocalibrate_p300_box.stateChanged.connect(p300_autocalibration_handle)
        self.blink_reaction.textChanged.connect(blink_reaction_text_updated)
        self.update_last_blink.connect(self._update_last_blink)
        self.update_ssvep_freq.connect(self._update_ssvep_freq)
        self.update_react_to.connect(self._update_react_to)

        # without this timer QtLoop will not ever give control to python to let it process signals
        # (except when clicking checkboxes which are connected to python functions)
        self._timer_for_shutdown = QTimer()
        self._timer_for_shutdown.timeout.connect(lambda: None)
        self._timer_for_shutdown.start(500)

    def _update_last_blink(self, s):
        self.latest_blink.setText(str(s))

    def _update_ssvep_freq(self, s):
        self.freq_box.setValue(s)

    def _update_react_to(self, s):
        self.blink_reaction.setPlainText(str(s))
