# -*- coding: utf-8 -*-
# Copyright (c) 2016-2018 Braintech Sp. z o.o. [Ltd.] <http://www.braintech.pl>
# All rights reserved.
import numpy
import numpy as np
from obci_p300.utils import SamplePacketCascadeFilter
from braintech.obci.signal_processing.signal.data_generic_write_proxy import SamplePacket
from scipy.signal import cheby2, iirnotch

VISUAL_DEBUG = False


def test_sample_cascade_filter():
    sfreq = 50000
    nyquist_rate = sfreq / 2
    timeline = numpy.arange(0, 10, 1 / sfreq)
    sin_pass = 20
    sin_damp = 100
    data = numpy.sin(2 * numpy.pi * sin_pass * timeline) + numpy.sin(2 * numpy.pi * sin_damp * timeline)
    data_fft = numpy.fft.fft(data)

    b, a = cheby2(3, 12, 50 / nyquist_rate, btype='lowpass')

    w0 = 50 / nyquist_rate
    b_notch, a_notch = iirnotch(w0, Q=w0 / (5 / nyquist_rate))

    filter = SamplePacketCascadeFilter([{'b': b, 'a': a}, {'b': b_notch, 'a': a_notch}])

    samples_in_package = 16
    new_data = []
    for i in range(0, len(timeline), samples_in_package):
        fragment = data[i:i + samples_in_package]
        packet = SamplePacket(samples=numpy.array([fragment, fragment]).T, ts=timeline[i:i + samples_in_package])
        filtered = filter.get(packet)
        new_data.extend(filtered.samples[:, 0])

    data_new_fft = numpy.fft.fft(new_data)
    freqs = numpy.fft.fftfreq(len(new_data)) * sfreq

    pass_mask = np.logical_and(freqs > (sin_pass - 0.1), freqs < (sin_pass + 0.1))
    damp_mask = np.logical_and(freqs > (sin_damp - 0.1), freqs < (sin_damp + 0.1))

    assert np.sum(np.abs(data_new_fft[pass_mask])) / np.sum(np.abs(data_fft[pass_mask])) > 0.9  # pass band passes
    assert np.sum(np.abs(data_new_fft[damp_mask])) / np.sum(np.abs(data_fft[damp_mask])) < 0.5  # damping band is damped
    assert np.all(np.abs(np.diff(new_data)) < 0.05)  # data is continious

    if VISUAL_DEBUG:
        import pylab as pb
        # data before and after filtering
        pb.plot(data)
        pb.plot(new_data)
        pb.figure()
        # figure 2 - FFT of signal before and after filtering
        pb.plot(freqs, np.abs(data_fft))
        pb.plot(freqs, np.abs(data_new_fft))
        pb.plot(freqs, pass_mask * 3000)
        pb.plot(freqs, damp_mask * 3000)
        pb.figure()
        # figure 3 - difference between neighbour samples - is signal continious?
        pb.plot(np.abs(np.diff(new_data)))
        pb.show()
