#!/usr/bin/env python3
# Copyright (c) 2016-2018 Braintech Sp. z o.o. [Ltd.] <http://www.braintech.pl>
# All rights reserved.

import sys
import numpy
import time
from collections import namedtuple
from braintech.obci.core.broker import messages
from braintech.obci.core.drivers.eeg.eeg_amplifier import SamplePacket

packet_t = namedtuple('Packet_t', ['samples', 'ts'])


class _TestAmplifierSerializationMsg(messages.SignalMessage):
    pass


class SamplesBenchmark:
    def __init__(self, samp_num=100000, channels_num=25, packets_num=100):

        print("Samples:  ", samp_num, " channels: ", channels_num, "packets: ", packets_num, )

        self.msg_type = _TestAmplifierSerializationMsg

        self.num_of_samples = samp_num
        self.num_of_channels = channels_num
        self.packets_num = packets_num
        self.packet = SamplePacket(samples=numpy.zeros((samp_num, channels_num)),
                                   ts=numpy.zeros(samp_num)
                                   )

    def create_msg(self, packet):
        msg_to_ser = _TestAmplifierSerializationMsg(sender='test_amp', data=packet)
        return msg_to_ser.serialize()

    def perform_packing_test(self):
        start_time = time.time()
        print("Start packing test: ")

        for p in range(self.packets_num):
            msg = self.create_msg(self.packet)

        end_time = time.time()
        print("End of packing test - time: ", end_time - start_time,
              " approx. sample rate: ", float(self.num_of_samples * self.packets_num) / (end_time - start_time))
        data_size = (len(msg[0]) + len(msg[1])) * self.packets_num
        print(float(data_size) / 1024 / 1024, " MiB")
        print(float(data_size) / (end_time - start_time) / 1024 / 1024 * 8, " Mbps")

    def perform_unpacking_test(self):
        msg = self.create_msg(self.packet)
        start_time = time.time()
        print("Start deserializing test: ")
        for p in range(self.packets_num):
            messages.deserialize(msg)
        end_time = time.time()
        print("End of unpacking test - time: ", end_time - start_time,
              " approx. sample rate: ",
              float(self.num_of_samples * self.packets_num) / (end_time - start_time))
        data_size = (len(msg[0]) + len(msg[1])) * self.packets_num
        print(float(data_size) / 1024 / 1024, " MiB")
        print(float(data_size) / (end_time - start_time) / 1000 / 1000 * 8, " Mbps")


if __name__ == "__main__":
    if len(sys.argv) != 4:
        print("Usage: ./sample_packaging_benchmark <num samples> <num channels> <num packets>")
        sys.exit(0)

    samp_num = int(sys.argv[1])
    chan_num = int(sys.argv[2])
    packets_num = int(sys.argv[3])

    b = SamplesBenchmark(samp_num=samp_num,
                         channels_num=chan_num,
                         packets_num=packets_num,
                         )

    b.perform_packing_test()
    b.perform_unpacking_test()
