# Copyright (c) 2016-2018 Braintech Sp. z o.o. [Ltd.] <http://www.braintech.pl>
# All rights reserved

# -*- coding: utf-8 -*-
# Copyright (c) 2016-2018 Braintech Sp. z o.o. [Ltd.] <http://www.braintech.pl>
# All rights reserved.
import glob
import os
import sys
import time
from collections import OrderedDict
from pathlib import Path

from devops.devpi import _get_package_version, _index_arg
from fabric import api, context_managers
from fabric.context_managers import lcd

try:
    import devops  # noqa
except ImportError:
    print("devops module not found! Not all commands are available!\n"
          "Install devops module using https://pip.braintech.pl/obci/stable", file=sys.stderr)
else:
    from devops.ci import install
    from devops.ci import output
    from devops import devpi

    sys.stdout = output.unbuffered_stdout

APT_REQUIREMENTS_PACKAGES = {
    'common': ['git'],
    'codestyle': [],
    'test': ['udev'],
    'build': ['gcc', 'libbluetooth3', 'libbluetooth-dev', 'libftdi-dev', 'libffi-dev']
}
PIP_REQUIREMENTS = {
    'build': ['setuptools>=20', 'cython'],
    'codestyle': [
        'flake8~=3.0.4',
        'flake8_docstrings~=1.0.2',
        'pydocstyle~=1.0.0',
    ],
    'test': [
        'pytest>=3.0',
        'pytest-timeout>=1.0',
        'pytest-mock~=1.10.4',
        'braintech-obci-experiment~=2.8.0',  # required for all tests to run, not required for amp per se
    ]
}


ESSENTIAL_PACKAGES = OrderedDict([
    ('braintech.drivers.native_amplifier_lib', 'braintech-drivers-native-amplifier-lib'),
    ]
)

PACKAGES = OrderedDict([
    ('braintech.drivers.native_amplifier_lib', 'braintech-drivers-native-amplifier-lib'),
    ('braintech.drivers.perun32', 'braintech-drivers-perun32'),
    ('braintech.drivers.perun8', 'braintech-drivers-perun8'),
    ('braintech.drivers.double_amplifier', 'braintech-drivers-double-amplifier'),
])



def setup_build_dependencies():
    for package, dir in ESSENTIAL_PACKAGES.items():
        # -e is needed due to versioneer bug
        install.local('pip3 install -e %s/.' % dir)


def setup_stage(name):
    install.setup_stage(name, APT_REQUIREMENTS_PACKAGES, PIP_REQUIREMENTS)
    if name == 'test':
        for package, dir in PACKAGES.items():
            for package_wheel in glob.glob('dist/{}*.whl'.format(dir.replace('-', '_'))):
                    install.local("pip3 install {}".format(package_wheel))
        install.local("install_all")


def install_editable():
    for package, dir in PACKAGES.items():
        install.local('pip3 install -e %s/.' % dir)


def install_packages():
    for package, dir in PACKAGES.items():
        install.local('pip3 install %s' % dir)


def upload_no_delete(path='.', index=None):
    # windows building deletes linux packages and vice versa
    cmd = 'devpi upload {index}'.format(
        index=_index_arg(index)
    )
    with context_managers.lcd(path):
        api.local(cmd)


def build_packages():
    setup_build_dependencies()
    Path('dist').mkdir(exist_ok=True)
    for package, dir in PACKAGES.items():
        upload_no_delete(dir)
        if dir != '.':
            api.local('mv {}/dist/* ./dist/'.format(dir))


def _push_no_delete(path, index, from_index=None):
    package, version = _get_package_version(path)
    with context_managers.lcd(path):
        api.local("devpi push {package}=={version} {index} {from_index}".format(
            package=package, index=index, version=version, from_index=_index_arg(from_index)
        ))


def deploy_packages():
    for package, dir in PACKAGES.items():
        with lcd(dir):
            api.local("devpi_push --from_tmp")
        devpi_index = devpi.current_index()['name']
        from_index = devpi.temp_name(devpi_index)
        _push_no_delete(dir, 'obci/installers', from_index)


def build_packages_without_remove():
    setup_build_dependencies()
    Path('dist').mkdir(exist_ok=True)
    print("Waiting for build_packages job to finish....")
    time.sleep(60)
    for package, dir in PACKAGES.items():
        with lcd(dir):
            api.local("devpi upload")
        if dir != '.':
            api.local('mv {}/dist/* ./dist/'.format(dir))
